@@ -35,6 +35,7 @@ import pcluster.cli.model
 from pcluster.api import encoder
 from pcluster.cli.commands.common import CliCommand, exit_msg, to_bool, to_int, to_number
 from pcluster.cli.exceptions import APIOperationException, ParameterException
+from pcluster.cli.logger import redirect_stdouterr_to_logger
 from pcluster.cli.middleware import add_additional_args, middleware_hooks
 from pcluster.utils import to_camel_case, to_snake_case
 
@@ -135,9 +136,21 @@ def gen_parser(model):
             else:
                 type_coerce = None
 
+            abbrev_args = {
+                "cluster-name": "-n",
+                "image-id": "-i",
+                "region": "-r",
+                "cluster-configuration": "-c",
+                "image-configuration": "-c",
+            }
+            if param["name"] in abbrev_args:
+                arg_name = [abbrev_args[param["name"]], f"--{param['name']}"]
+            else:
+                arg_name = [f"--{param['name']}"]
+
             # add the parameter to the parser based on type from model / specification
             subparser.add_argument(
-                f"--{param['name']}",
+                *arg_name,
                 required=param.get("required", False),
                 choices=param.get("enum", None),
                 nargs="+" if "multi" in param else None,
@@ -167,7 +180,8 @@ def add_cli_commands(parser_map):
 def _run_operation(model, args, extra_args):
     if args.operation in model:
         try:
-            return args.func(args)
+            with redirect_stdouterr_to_logger():
+                return args.func(args)
         except KeyboardInterrupt as e:
             raise e
         except APIOperationException as e:
