@@ -30,6 +30,7 @@ from jinja2 import BaseLoader
 from jinja2.sandbox import SandboxedEnvironment
 from marshmallow import ValidationError
 
+from pcluster.api.models import Metadata
 from pcluster.aws.aws_api import AWSApi
 from pcluster.aws.common import AWSClientError, BadRequestError, LimitExceededError, StackNotFoundError, get_region
 from pcluster.config.cluster_config import BaseClusterConfig, SchedulerPluginScheduling, SlurmScheduling, Tag
@@ -191,6 +192,11 @@ class Cluster:
             self._get_artifact_dir()
         return self.__s3_artifact_dir
 
+    @property
+    def compute_fleet_status_manager(self) -> ComputeFleetStatusManager:
+        """Return compute fleet status manager."""
+        return ComputeFleetStatusManager.get_manager(self.name, self.stack.version, self.stack.scheduler)
+
     def _get_artifact_dir(self):
         """Get artifact directory in S3 bucket by stack output."""
         try:
@@ -272,7 +278,7 @@ class Cluster:
         return status
 
     @property
-    def compute_fleet_status_with_last_updated_time(self):
+    def compute_fleet_status_with_last_updated_time(self) -> Tuple[ComputeFleetStatus, str]:
         """Status of the cluster compute fleet and the last compute fleet status updated time."""
         if self.stack.is_working_status or self.stack.status == "UPDATE_IN_PROGRESS":
             if self.stack.scheduler == "awsbatch":
@@ -281,8 +287,7 @@ class Cluster:
                 )
                 last_updated_time = None
             else:
-                compute_fleet_status_manager = ComputeFleetStatusManager(self.name)
-                status, last_updated_time = compute_fleet_status_manager.get_status_with_last_updated_time()
+                status, last_updated_time = self.compute_fleet_status_manager.get_status_with_last_updated_time()
             return status, last_updated_time
         else:
             LOGGER.info(
@@ -347,7 +352,7 @@ class Cluster:
                 validator_suppressors, validation_failure_level
             )
 
-            self._add_version_tag()
+            self._add_tags()
             self._generate_artifact_dir()
             artifact_dir_generated = True
             self._upload_config()
@@ -458,12 +463,28 @@ class Cluster:
 
                 # original config version will be stored in CloudFormation Parameters
                 self.config.original_config_version = result.get("VersionId")
-
         except Exception as e:
             raise _cluster_error_mapper(
                 e, f"Unable to upload cluster config to the S3 bucket {self.bucket.name} due to exception: {e}"
             )
 
+    def _upload_change_set(self, changes=None):
+        """Upload change set."""
+        if changes:
+            self._check_bucket_existence()
+            try:
+                self.bucket.upload_config(
+                    config=ConfigPatch.generate_json_change_set(changes),
+                    config_name=PCLUSTER_S3_ARTIFACTS_DICT.get("change_set_name"),
+                    format=S3FileFormat.JSON,
+                )
+            except Exception as e:
+                message = (
+                    f"Unable to upload cluster change set to the S3 bucket {self.bucket.name} due to exception: {e}"
+                )
+                LOGGER.error(message)
+                raise _cluster_error_mapper(e, message)
+
     def _upload_artifacts(self):
         """
         Upload cluster specific resources and cluster template.
@@ -518,7 +539,13 @@ class Cluster:
             if scheduler_plugin_template.startswith("s3"):
                 bucket_parsing_result = parse_bucket_url(scheduler_plugin_template)
                 result = AWSApi.instance().s3.get_object(
-                    bucket_name=bucket_parsing_result["bucket_name"], key=bucket_parsing_result["object_key"]
+                    bucket_name=bucket_parsing_result["bucket_name"],
+                    key=bucket_parsing_result["object_key"],
+                    expected_bucket_owner=get_attr(
+                        self.config,
+                        "scheduling.settings.scheduler_definition.cluster_infrastructure.cloud_formation."
+                        "s3_bucket_owner",
+                    ),
                 )
                 file_content = result["Body"].read().decode("utf-8")
             else:
@@ -531,6 +558,9 @@ class Cluster:
                 f"Error while downloading scheduler plugin artifacts from '{scheduler_plugin_template}': {str(e)}"
             ) from e
 
+        # checksum
+        self.validate_scheduler_plugin_template_checksum(file_content, scheduler_plugin_template)
+
         # jinja rendering
         try:
             LOGGER.info("Rendering the following scheduler plugin CloudFormation template:\n%s", file_content)
@@ -677,15 +707,15 @@ class Cluster:
         except AWSClientError as e:
             raise _cluster_error_mapper(e, f"Failed to retrieve cluster instances. {e}")
 
-    def has_running_capacity(self, updated_value: bool = False):
+    def has_running_capacity(self, updated_value: bool = False) -> bool:
         """Return True if the cluster has running capacity. Note: the value will be cached."""
         if self.__has_running_capacity is None or updated_value:
-            if self.stack.scheduler == "slurm":
-                self.__has_running_capacity = (
-                    ComputeFleetStatusManager(self.name).get_status() != ComputeFleetStatus.STOPPED
-                )
-            elif self.stack.scheduler == "awsbatch":
+            if self.stack.scheduler == "awsbatch":
                 self.__has_running_capacity = self.get_running_capacity() > 0
+            else:
+                self.__has_running_capacity = (
+                    self.compute_fleet_status_manager.get_status() != ComputeFleetStatus.STOPPED
+                )
         return self.__has_running_capacity
 
     def get_running_capacity(self, updated_value: bool = False):
@@ -710,8 +740,8 @@ class Cluster:
             scheduler = self.config.scheduling.scheduler
             if scheduler == "awsbatch":
                 self.enable_awsbatch_compute_environment()
-            else:  # scheduler == "slurm"
-                self.start_slurm_compute_fleet()
+            else:  # traditional scheduler
+                self.start_compute_fleet()
         except ComputeFleetStatusManager.ConditionalStatusUpdateFailed:
             raise BadRequestClusterActionError(
                 "Failed when starting compute fleet due to a concurrent update of the status. "
@@ -720,10 +750,9 @@ class Cluster:
         except Exception as e:
             raise _cluster_error_mapper(e, f"Failed when starting compute fleet with error: {str(e)}")
 
-    def start_slurm_compute_fleet(self):
-        """Start Slurm compute fleet."""
-        compute_fleet_status_manager = ComputeFleetStatusManager(self.name)
-        compute_fleet_status_manager.update_status(
+    def start_compute_fleet(self):
+        """Start compute fleet."""
+        self.compute_fleet_status_manager.update_status(
             ComputeFleetStatus.START_REQUESTED, ComputeFleetStatus.STARTING, ComputeFleetStatus.RUNNING
         )
 
@@ -753,8 +782,8 @@ class Cluster:
             scheduler = self.config.scheduling.scheduler
             if scheduler == "awsbatch":
                 self.disable_awsbatch_compute_environment()
-            else:  # scheduler == "slurm"
-                self.stop_slurm_compute_fleet()
+            else:  # traditional scheduler
+                self.stop_compute_fleet()
         except ComputeFleetStatusManager.ConditionalStatusUpdateFailed:
             raise BadRequestClusterActionError(
                 "Failed when stopping compute fleet due to a concurrent update of the status. "
@@ -763,10 +792,9 @@ class Cluster:
         except Exception as e:
             raise _cluster_error_mapper(e, f"Failed when stopping compute fleet with error: {str(e)}")
 
-    def stop_slurm_compute_fleet(self):
-        """Stop Slurm compute fleet."""
-        compute_fleet_status_manager = ComputeFleetStatusManager(self.name)
-        compute_fleet_status_manager.update_status(
+    def stop_compute_fleet(self):
+        """Stop compute fleet."""
+        self.compute_fleet_status_manager.update_status(
             ComputeFleetStatus.STOP_REQUESTED, ComputeFleetStatus.STOPPING, ComputeFleetStatus.STOPPED
         )
 
@@ -843,8 +871,9 @@ class Cluster:
             self.config = target_config
             self.__source_config_text = target_source_config
 
-            self._add_version_tag()
+            self._add_tags()
             self._upload_config()
+            self._upload_change_set(changes)
 
             # Create template if not provided by the user
             if not (self.config.dev_settings and self.config.dev_settings.cluster_template):
@@ -880,14 +909,19 @@ class Cluster:
             LOGGER.critical(e)
             raise _cluster_error_mapper(e, f"Cluster update failed.\n{e}")
 
-    def _add_version_tag(self):
-        """Add version tag to the stack."""
+    def _add_tags(self):
+        """Add tags PCLUSTER_CLUSTER_NAME_TAG and PCLUSTER_VERSION_TAG to the attribute config.tags."""
         if self.config.tags is None:
             self.config.tags = []
-        # Remove PCLUSTER_VERSION_TAG if already exists
-        self.config.tags = [tag for tag in self.config.tags if tag.key != PCLUSTER_VERSION_TAG]
-        # Add PCLUSTER_VERSION_TAG
-        self.config.tags.append(Tag(key=PCLUSTER_VERSION_TAG, value=get_installed_version()))
+
+        # List of tags to be added to the cluster
+        tags = {PCLUSTER_CLUSTER_NAME_TAG: self.name, PCLUSTER_VERSION_TAG: get_installed_version()}
+
+        # Remove tags if they already exist
+        self.config.tags = [tag for tag in self.config.tags if tag.key not in tags]
+
+        # Add the tags
+        self.config.tags += [Tag(key=tag_key, value=tag_value) for tag_key, tag_value in tags.items()]
 
     def _get_cfn_tags(self):
         """Return tag list in the format expected by CFN."""
@@ -906,7 +940,7 @@ class Cluster:
         keep_s3_objects: bool = False,
         start_time: datetime = None,
         end_time: datetime = None,
-        filters: str = None,
+        filters: List[str] = None,
         output_file: str = None,
     ):
         """
@@ -919,7 +953,7 @@ class Cluster:
         :param keep_s3_objects: Keep the exported objects exports to S3. The default behavior is to delete them
         :param start_time: Start time of interval of interest for log events. ISO 8601 format: YYYY-MM-DDThh:mm:ssTZD
         :param end_time: End time of interval of interest for log events. ISO 8601 format: YYYY-MM-DDThh:mm:ssTZD
-        :param filters: Filters in the format Name=name,Values=value1,value2
+        :param filters: Filters in the format ["Name=name,Values=value1,value2"]
                Accepted filters are: private_dns_name, node_type==HeadNode
         """
         # check stack
@@ -1151,3 +1185,21 @@ class Cluster:
                     "scheduling configuration.",
                     [changes[0]] + scheduling_changes,
                 )
+
+    def validate_scheduler_plugin_template_checksum(self, file_content, scheduler_plugin_template):
+        """Validate scheduler plugin template checksum match the expected checksum."""
+        checksum = get_attr(
+            self.config, "scheduling.settings.scheduler_definition.cluster_infrastructure.cloud_formation.checksum"
+        )
+        if checksum:
+            actual_checksum = hashlib.sha256(file_content.encode()).hexdigest()
+            if actual_checksum != checksum:
+                raise BadRequestClusterActionError(
+                    f"Error when validating scheduler plugin template '{scheduler_plugin_template}': "
+                    f"checksum: {actual_checksum} does not match expected one: {checksum}"
+                )
+
+    def get_plugin_metadata(self):
+        """Get the metadata name and version used for the response of DescribeCluster when the scheduler is plugin."""
+        full_metadata = get_attr(self.config, "scheduling.settings.scheduler_definition.metadata")
+        return Metadata(name=full_metadata.get("Name"), version=full_metadata.get("Version")) if full_metadata else None
