@@ -15,11 +15,12 @@
 # This module contains all the classes required to convert a Cluster into a CFN template by using CDK.
 #
 import json
-from collections import namedtuple
+from collections import defaultdict, namedtuple
 from datetime import datetime
 from typing import Dict, List, Union
 
 from aws_cdk import aws_cloudformation as cfn
+from aws_cdk import aws_dynamodb as dynamomdb
 from aws_cdk import aws_ec2 as ec2
 from aws_cdk import aws_efs as efs
 from aws_cdk import aws_fsx as fsx
@@ -28,6 +29,7 @@ from aws_cdk import aws_lambda as awslambda
 from aws_cdk import aws_logs as logs
 from aws_cdk.core import (
     CfnCustomResource,
+    CfnDeletionPolicy,
     CfnOutput,
     CfnParameter,
     CfnResource,
@@ -42,19 +44,28 @@ from aws_cdk.core import (
 from pcluster.aws.aws_api import AWSApi
 from pcluster.config.cluster_config import (
     AwsBatchClusterConfig,
+    BaseSharedFsx,
     CapacityType,
+    ExistingFsxOntap,
+    ExistingFsxOpenZfs,
+    SchedulerPluginQueue,
     SharedEbs,
     SharedEfs,
-    SharedFsx,
+    SharedFsxLustre,
     SharedStorageType,
     SlurmClusterConfig,
+    SlurmQueue,
 )
 from pcluster.constants import (
     CW_LOG_GROUP_NAME_PREFIX,
     CW_LOGS_CFN_PARAM_NAME,
+    DEFAULT_EPHEMERAL_DIR,
+    LUSTRE,
+    NODE_BOOTSTRAP_TIMEOUT,
     OS_MAPPING,
     PCLUSTER_CLUSTER_NAME_TAG,
     PCLUSTER_COMPUTE_RESOURCE_NAME_TAG,
+    PCLUSTER_DYNAMODB_PREFIX,
     PCLUSTER_QUEUE_NAME_TAG,
     PCLUSTER_S3_ARTIFACTS_DICT,
 )
@@ -81,9 +92,9 @@ from pcluster.templates.cdk_builder_utils import (
     get_log_group_deletion_policy,
     get_queue_security_groups_full,
     get_shared_storage_ids_by_type,
-    get_shared_storage_options_by_type,
     get_slurm_specific_dna_json_for_head_node,
     get_user_data_content,
+    to_comma_separated_string,
 )
 from pcluster.templates.cw_dashboard_builder import CWDashboardConstruct
 from pcluster.templates.slurm_builder import SlurmConstruct
@@ -120,9 +131,9 @@ class ClusterCdkStack(Stack):
                 timestamp = f"{datetime.utcnow().strftime('%Y%m%d%H%M')}"
                 self.log_group_name = f"{CW_LOG_GROUP_NAME_PREFIX}{self.stack_name}-{timestamp}"
 
-        self.shared_storage_mappings = {storage_type: [] for storage_type in SharedStorageType}
-        self.shared_storage_options = {storage_type: "" for storage_type in SharedStorageType}
-        self.shared_storage_attributes = {storage_type: {} for storage_type in SharedStorageType}
+        self.shared_storage_infos = {storage_type: [] for storage_type in SharedStorageType}
+        self.shared_storage_mount_dirs = {storage_type: [] for storage_type in SharedStorageType}
+        self.shared_storage_attributes = {storage_type: defaultdict(list) for storage_type in SharedStorageType}
 
         self._add_parameters()
         self._add_resources()
@@ -239,46 +250,16 @@ class ClusterCdkStack(Stack):
             for storage in self.config.shared_storage:
                 self._add_shared_storage(storage)
 
-        # Compute Fleet and scheduler related resources
-        self.scheduler_resources = None
-        if self._condition_is_slurm():
-            self.scheduler_resources = SlurmConstruct(
-                scope=self,
-                id="Slurm",
-                stack_name=self._stack_name,
-                cluster_config=self.config,
-                bucket=self.bucket,
-                managed_head_node_instance_role=self._managed_head_node_instance_role,
-                managed_compute_instance_roles=self._managed_compute_instance_roles,
-                cleanup_lambda_role=cleanup_lambda_role,  # None if provided by the user
-                cleanup_lambda=cleanup_lambda,
-            )
-        self.compute_fleet_resources = None
-        if not self._condition_is_batch():
-            self.compute_fleet_resources = ComputeFleetConstruct(
-                scope=self,
-                id="ComputeFleet",
-                cluster_config=self.config,
-                log_group=self.log_group,
-                cleanup_lambda=cleanup_lambda,
-                cleanup_lambda_role=cleanup_lambda_role,
-                compute_security_group=self._compute_security_group,
-                shared_storage_mappings=self.shared_storage_mappings,
-                shared_storage_options=self.shared_storage_options,
-                shared_storage_attributes=self.shared_storage_attributes,
-                compute_node_instance_profiles=self._compute_instance_profiles,
-                cluster_hosted_zone=self.scheduler_resources.cluster_hosted_zone if self.scheduler_resources else None,
-                dynamodb_table=self.scheduler_resources.dynamodb_table if self.scheduler_resources else None,
-                head_eni=self._head_eni,
-            )
-
-        self._add_scheduler_plugin_substack()
+        self._add_fleet_and_scheduler_resources(cleanup_lambda, cleanup_lambda_role)
 
         # Wait condition
         self.wait_condition, self.wait_condition_handle = self._add_wait_condition()
 
         # Head Node
         self.head_node_instance = self._add_head_node()
+        # Add a dependency to the cleanup Route53 resource, so that Route53 Hosted Zone is cleaned after node is deleted
+        if self._condition_is_slurm() and hasattr(self.scheduler_resources, "cleanup_route53_custom_resource"):
+            self.head_node_instance.add_depends_on(self.scheduler_resources.cleanup_route53_custom_resource)
 
         # AWS Batch related resources
         if self._condition_is_batch():
@@ -290,8 +271,8 @@ class ClusterCdkStack(Stack):
                 bucket=self.bucket,
                 create_lambda_roles=self._condition_create_lambda_iam_role(),
                 compute_security_group=self._compute_security_group,
-                shared_storage_mappings=self.shared_storage_mappings,
-                shared_storage_options=self.shared_storage_options,
+                shared_storage_infos=self.shared_storage_infos,
+                shared_storage_mount_dirs=self.shared_storage_mount_dirs,
                 head_node_instance=self.head_node_instance,
                 managed_head_node_instance_role=self._managed_head_node_instance_role,  # None if provided by the user
             )
@@ -304,7 +285,7 @@ class ClusterCdkStack(Stack):
                 stack_name=self.stack_name,
                 cluster_config=self.config,
                 head_node_instance=self.head_node_instance,
-                shared_storage_mappings=self.shared_storage_mappings,
+                shared_storage_infos=self.shared_storage_infos,
                 cw_log_group_name=self.log_group.log_group_name if self.config.is_cw_logging_enabled else None,
             )
 
@@ -335,6 +316,55 @@ class ClusterCdkStack(Stack):
         log_group.cfn_options.deletion_policy = get_log_group_deletion_policy(self.config)
         return log_group
 
+    def _add_fleet_and_scheduler_resources(self, cleanup_lambda, cleanup_lambda_role):
+        # Compute Fleet and scheduler related resources
+        self.scheduler_resources = None
+        if self._condition_is_slurm():
+            self.scheduler_resources = SlurmConstruct(
+                scope=self,
+                id="Slurm",
+                stack_name=self._stack_name,
+                cluster_config=self.config,
+                bucket=self.bucket,
+                managed_head_node_instance_role=self._managed_head_node_instance_role,
+                managed_compute_instance_roles=self._managed_compute_instance_roles,
+                cleanup_lambda_role=cleanup_lambda_role,  # None if provided by the user
+                cleanup_lambda=cleanup_lambda,
+            )
+        if not self._condition_is_batch():
+            _dynamodb_table_status = dynamomdb.CfnTable(
+                self,
+                "DynamoDBTable",
+                table_name=PCLUSTER_DYNAMODB_PREFIX + self.stack_name,
+                attribute_definitions=[
+                    dynamomdb.CfnTable.AttributeDefinitionProperty(attribute_name="Id", attribute_type="S"),
+                ],
+                key_schema=[dynamomdb.CfnTable.KeySchemaProperty(attribute_name="Id", key_type="HASH")],
+                billing_mode="PAY_PER_REQUEST",
+            )
+            _dynamodb_table_status.cfn_options.update_replace_policy = CfnDeletionPolicy.RETAIN
+            _dynamodb_table_status.cfn_options.deletion_policy = CfnDeletionPolicy.DELETE
+            self.dynamodb_table_status = _dynamodb_table_status
+        self.compute_fleet_resources = None
+        if not self._condition_is_batch():
+            self.compute_fleet_resources = ComputeFleetConstruct(
+                scope=self,
+                id="ComputeFleet",
+                cluster_config=self.config,
+                log_group=self.log_group,
+                cleanup_lambda=cleanup_lambda,
+                cleanup_lambda_role=cleanup_lambda_role,
+                compute_security_group=self._compute_security_group,
+                shared_storage_infos=self.shared_storage_infos,
+                shared_storage_mount_dirs=self.shared_storage_mount_dirs,
+                shared_storage_attributes=self.shared_storage_attributes,
+                compute_node_instance_profiles=self._compute_instance_profiles,
+                cluster_hosted_zone=self.scheduler_resources.cluster_hosted_zone if self.scheduler_resources else None,
+                dynamodb_table=self.scheduler_resources.dynamodb_table if self.scheduler_resources else None,
+                head_eni=self._head_eni,
+            )
+        self._add_scheduler_plugin_substack()
+
     def _add_cleanup_resources_lambda(self):
         """Create Lambda cleanup resources function and its role."""
         cleanup_resources_lambda_role = None
@@ -566,25 +596,33 @@ class ClusterCdkStack(Stack):
 
     def _add_shared_storage(self, storage):
         """Add specific Cfn Resources to map the shared storage and store the output filesystem id."""
-        storage_ids_list = self.shared_storage_mappings[storage.shared_storage_type]
-        cfn_resource_id = "{0}{1}".format(storage.shared_storage_type.name, len(storage_ids_list))
+        storage_list = self.shared_storage_infos[storage.shared_storage_type]
+        cfn_resource_id = "{0}{1}".format(storage.shared_storage_type.name, len(storage_list))
         if storage.shared_storage_type == SharedStorageType.FSX:
-            storage_ids_list.append(StorageInfo(self._add_fsx_storage(cfn_resource_id, storage), storage))
+            storage_list.append(StorageInfo(self._add_fsx_storage(cfn_resource_id, storage), storage))
         elif storage.shared_storage_type == SharedStorageType.EBS:
-            storage_ids_list.append(StorageInfo(self._add_ebs_volume(cfn_resource_id, storage), storage))
+            storage_list.append(StorageInfo(self._add_ebs_volume(cfn_resource_id, storage), storage))
         elif storage.shared_storage_type == SharedStorageType.EFS:
-            storage_ids_list.append(StorageInfo(self._add_efs_storage(cfn_resource_id, storage), storage))
+            storage_list.append(StorageInfo(self._add_efs_storage(cfn_resource_id, storage), storage))
         elif storage.shared_storage_type == SharedStorageType.RAID:
-            storage_ids_list.extend(self._add_raid_volume(cfn_resource_id, storage))
+            storage_list.extend(self._add_raid_volume(cfn_resource_id, storage))
+        self.shared_storage_mount_dirs[storage.shared_storage_type].append(storage.mount_dir)
 
-    def _add_fsx_storage(self, id: str, shared_fsx: SharedFsx):
+    def _add_fsx_storage(self, id: str, shared_fsx: BaseSharedFsx):
         """Add specific Cfn Resources to map the FSX storage."""
-        fsx_id = shared_fsx.file_system_id
-        # Initialize DNSName and MountName for existing filesystem, if any
-        self.shared_storage_attributes[shared_fsx.shared_storage_type]["MountName"] = shared_fsx.existing_mount_name
-        self.shared_storage_attributes[shared_fsx.shared_storage_type]["DNSName"] = shared_fsx.existing_dns_name
-
-        if not fsx_id and shared_fsx.mount_dir:
+        fsx_id = shared_fsx.file_system_id if isinstance(shared_fsx, SharedFsxLustre) else shared_fsx.volume_id
+        mount_name = ""
+        dns_name = ""
+        volume_junction_path = ""
+        if fsx_id:
+            dns_name = shared_fsx.existing_dns_name
+            if isinstance(shared_fsx, ExistingFsxOntap):
+                volume_junction_path = shared_fsx.junction_path
+            if isinstance(shared_fsx, ExistingFsxOpenZfs):
+                volume_junction_path = shared_fsx.volume_path
+            if isinstance(shared_fsx, SharedFsxLustre):
+                mount_name = shared_fsx.existing_mount_name
+        else:
             # Drive cache type must be set for HDD (Either "NONE" or "READ"), and must not be set for SDD (None).
             drive_cache_type = None
             if shared_fsx.fsx_storage_type == "HDD":
@@ -612,47 +650,24 @@ class ClusterCdkStack(Stack):
                 ),
                 backup_id=shared_fsx.backup_id,
                 kms_key_id=shared_fsx.kms_key_id,
-                file_system_type="LUSTRE",
+                file_system_type=LUSTRE,
                 storage_type=shared_fsx.fsx_storage_type,
                 subnet_ids=self.config.compute_subnet_ids,
                 security_group_ids=self._get_compute_security_groups(),
+                file_system_type_version=shared_fsx.file_system_type_version,
                 tags=[CfnTag(key="Name", value=shared_fsx.name)],
             )
             fsx_id = fsx_resource.ref
-            # Get MountName for new filesystem
-            # DNSName cannot be retrieved from CFN and will be generated in cookbook
-            self.shared_storage_attributes[shared_fsx.shared_storage_type][
-                "MountName"
-            ] = fsx_resource.attr_lustre_mount_name
+            # Get MountName for new filesystem. DNSName cannot be retrieved from CFN and will be generated in cookbook
+            mount_name = fsx_resource.attr_lustre_mount_name
 
-        # [shared_dir,fsx_fs_id,storage_capacity,fsx_kms_key_id,imported_file_chunk_size,
-        # export_path,import_path,weekly_maintenance_start_time,deployment_type,
-        # per_unit_storage_throughput,daily_automatic_backup_start_time,
-        # automatic_backup_retention_days,copy_tags_to_backups,fsx_backup_id,
-        # auto_import_policy,storage_type,drive_cache_type,existing_mount_name,existing_dns_name]",
-        self.shared_storage_options[shared_fsx.shared_storage_type] = ",".join(
-            str(item)
-            for item in [
-                shared_fsx.mount_dir,
-                fsx_id,
-                shared_fsx.storage_capacity or "NONE",
-                shared_fsx.kms_key_id or "NONE",
-                shared_fsx.imported_file_chunk_size or "NONE",
-                shared_fsx.export_path or "NONE",
-                shared_fsx.import_path or "NONE",
-                shared_fsx.weekly_maintenance_start_time or "NONE",
-                shared_fsx.deployment_type or "NONE",
-                shared_fsx.per_unit_storage_throughput or "NONE",
-                shared_fsx.daily_automatic_backup_start_time or "NONE",
-                shared_fsx.automatic_backup_retention_days or "NONE",
-                shared_fsx.copy_tags_to_backups if shared_fsx.copy_tags_to_backups is not None else "NONE",
-                shared_fsx.backup_id or "NONE",
-                shared_fsx.auto_import_policy or "NONE",
-                shared_fsx.fsx_storage_type or "NONE",
-                shared_fsx.drive_cache_type or "NONE",
-                shared_fsx.existing_mount_name,
-                shared_fsx.existing_dns_name,
-            ]
+        self.shared_storage_attributes[shared_fsx.shared_storage_type]["DNSNames"].append(dns_name)
+        self.shared_storage_attributes[shared_fsx.shared_storage_type]["MountNames"].append(mount_name)
+        self.shared_storage_attributes[shared_fsx.shared_storage_type]["VolumeJunctionPaths"].append(
+            volume_junction_path
+        )
+        self.shared_storage_attributes[shared_fsx.shared_storage_type]["FileSystemTypes"].append(
+            shared_fsx.file_system_type
         )
 
         return fsx_id
@@ -696,22 +711,6 @@ class ClusterCdkStack(Stack):
             new_file_system,
         )
 
-        # [shared_dir,efs_fs_id,performance_mode,efs_kms_key_id,provisioned_throughput,encrypted,
-        # throughput_mode,exists_valid_head_node_mt,exists_valid_compute_mt]
-        self.shared_storage_options[shared_efs.shared_storage_type] = ",".join(
-            str(item)
-            for item in [
-                shared_efs.mount_dir,
-                efs_id,
-                shared_efs.performance_mode or "NONE",
-                shared_efs.kms_key_id or "NONE",
-                shared_efs.provisioned_throughput or "NONE",
-                shared_efs.encrypted if shared_efs.encrypted is not None else "NONE",
-                shared_efs.throughput_mode or "NONE",
-                "NONE",  # Useless
-                "NONE",  # Useless
-            ]
-        )
         return efs_id
 
     def _add_efs_mount_target(
@@ -742,22 +741,7 @@ class ClusterCdkStack(Stack):
         for index in range(shared_ebs.raid.number_of_volumes):
             ebs_ids.append(StorageInfo(self._add_cfn_volume(f"{id_prefix}Volume{index}", shared_ebs), shared_ebs))
 
-        # [shared_dir,raid_type,num_of_raid_volumes,volume_type,volume_size,volume_iops,encrypted,
-        # ebs_kms_key_id,volume_throughput]
-        self.shared_storage_options[shared_ebs.shared_storage_type] = ",".join(
-            str(item)
-            for item in [
-                shared_ebs.mount_dir,
-                shared_ebs.raid.raid_type,
-                shared_ebs.raid.number_of_volumes,
-                shared_ebs.volume_type,
-                shared_ebs.size,
-                shared_ebs.iops,
-                shared_ebs.encrypted if shared_ebs.encrypted is not None else "NONE",
-                shared_ebs.kms_key_id or "NONE",
-                shared_ebs.throughput,
-            ]
-        )
+        self.shared_storage_attributes[shared_ebs.shared_storage_type]["Type"] = str(shared_ebs.raid.raid_type)
 
         return ebs_ids
 
@@ -766,14 +750,6 @@ class ClusterCdkStack(Stack):
         ebs_id = shared_ebs.volume_id
         if not ebs_id and shared_ebs.mount_dir:
             ebs_id = self._add_cfn_volume(id, shared_ebs)
-
-        # Append mount dir to list of shared dirs
-        self.shared_storage_options[shared_ebs.shared_storage_type] += (
-            f",{shared_ebs.mount_dir}"
-            if self.shared_storage_options[shared_ebs.shared_storage_type]
-            else f"{shared_ebs.mount_dir}"
-        )
-
         return ebs_id
 
     def _add_cfn_volume(self, id: str, shared_ebs: SharedEbs):
@@ -796,7 +772,13 @@ class ClusterCdkStack(Stack):
     def _add_wait_condition(self):
         wait_condition_handle = cfn.CfnWaitConditionHandle(self, id="HeadNodeWaitConditionHandle" + self.timestamp)
         wait_condition = cfn.CfnWaitCondition(
-            self, id="HeadNodeWaitCondition" + self.timestamp, count=1, handle=wait_condition_handle.ref, timeout="1800"
+            self,
+            id="HeadNodeWaitCondition" + self.timestamp,
+            count=1,
+            handle=wait_condition_handle.ref,
+            timeout=str(
+                get_attr(self.config, "dev_settings.timeouts.head_node_bootstrap_timeout", NODE_BOOTSTRAP_TIMEOUT)
+            ),
         )
         if self.scheduler_plugin_stack:
             wait_condition.add_depends_on(self.scheduler_plugin_stack)
@@ -850,7 +832,7 @@ class ClusterCdkStack(Stack):
                     ec2.CfnLaunchTemplate.TagSpecificationProperty(
                         resource_type="instance",
                         tags=get_default_instance_tags(
-                            self._stack_name, self.config, head_node, "HeadNode", self.shared_storage_mappings
+                            self._stack_name, self.config, head_node, "HeadNode", self.shared_storage_infos
                         )
                         + get_custom_tags(self.config),
                     ),
@@ -878,11 +860,12 @@ class ClusterCdkStack(Stack):
                     "scheduler_plugin_substack_arn": self.scheduler_plugin_stack.ref
                     if self.scheduler_plugin_stack
                     else "",
-                    "raid_vol_ids": get_shared_storage_ids_by_type(
-                        self.shared_storage_mappings, SharedStorageType.RAID
+                    "raid_vol_ids": get_shared_storage_ids_by_type(self.shared_storage_infos, SharedStorageType.RAID),
+                    "raid_shared_dir": to_comma_separated_string(
+                        self.shared_storage_mount_dirs[SharedStorageType.RAID]
                     ),
-                    "raid_parameters": get_shared_storage_options_by_type(
-                        self.shared_storage_options, SharedStorageType.RAID
+                    "raid_type": to_comma_separated_string(
+                        self.shared_storage_attributes[SharedStorageType.RAID]["Type"]
                     ),
                     "disable_hyperthreading_manually": "true"
                     if head_node.disable_simultaneous_multithreading_manually
@@ -897,27 +880,32 @@ class ClusterCdkStack(Stack):
                     if post_install_action and post_install_action.args
                     else "NONE",
                     "region": self.region,
-                    "efs_fs_id": get_shared_storage_ids_by_type(self.shared_storage_mappings, SharedStorageType.EFS),
-                    "efs_shared_dir": get_shared_storage_options_by_type(
-                        self.shared_storage_options, SharedStorageType.EFS
-                    ),  # FIXME
-                    "fsx_fs_id": get_shared_storage_ids_by_type(self.shared_storage_mappings, SharedStorageType.FSX),
-                    "fsx_mount_name": self.shared_storage_attributes[SharedStorageType.FSX].get("MountName", ""),
-                    "fsx_dns_name": self.shared_storage_attributes[SharedStorageType.FSX].get("DNSName", ""),
-                    "fsx_options": get_shared_storage_options_by_type(
-                        self.shared_storage_options, SharedStorageType.FSX
+                    "efs_fs_ids": get_shared_storage_ids_by_type(self.shared_storage_infos, SharedStorageType.EFS),
+                    "efs_shared_dirs": to_comma_separated_string(self.shared_storage_mount_dirs[SharedStorageType.EFS]),
+                    "fsx_fs_ids": get_shared_storage_ids_by_type(self.shared_storage_infos, SharedStorageType.FSX),
+                    "fsx_mount_names": to_comma_separated_string(
+                        self.shared_storage_attributes[SharedStorageType.FSX]["MountNames"]
                     ),
-                    "volume": get_shared_storage_ids_by_type(self.shared_storage_mappings, SharedStorageType.EBS),
+                    "fsx_dns_names": to_comma_separated_string(
+                        self.shared_storage_attributes[SharedStorageType.FSX]["DNSNames"]
+                    ),
+                    "fsx_volume_junction_paths": to_comma_separated_string(
+                        self.shared_storage_attributes[SharedStorageType.FSX]["VolumeJunctionPaths"]
+                    ),
+                    "fsx_fs_types": to_comma_separated_string(
+                        self.shared_storage_attributes[SharedStorageType.FSX]["FileSystemTypes"]
+                    ),
+                    "fsx_shared_dirs": to_comma_separated_string(self.shared_storage_mount_dirs[SharedStorageType.FSX]),
+                    "volume": get_shared_storage_ids_by_type(self.shared_storage_infos, SharedStorageType.EBS),
                     "scheduler": self.config.scheduling.scheduler,
                     "ephemeral_dir": head_node.local_storage.ephemeral_volume.mount_dir
-                    if head_node.local_storage and head_node.local_storage.ephemeral_volume
-                    else "/scratch",
-                    "ebs_shared_dirs": get_shared_storage_options_by_type(
-                        self.shared_storage_options, SharedStorageType.EBS
-                    ),
+                    if head_node.local_storage.ephemeral_volume
+                    else DEFAULT_EPHEMERAL_DIR,
+                    "ebs_shared_dirs": to_comma_separated_string(self.shared_storage_mount_dirs[SharedStorageType.EBS]),
                     "proxy": head_node.networking.proxy.http_proxy_address if head_node.networking.proxy else "NONE",
                     "node_type": "HeadNode",
                     "cluster_user": OS_MAPPING[self.config.image.os]["user"],
+                    "ddb_table": self.dynamodb_table_status.ref if not self._condition_is_batch() else "NONE",
                     "log_group_name": self.log_group.log_group_name
                     if self.config.monitoring.logs.cloud_watch.enabled
                     else "NONE",
@@ -930,10 +918,16 @@ class ClusterCdkStack(Stack):
                         self.bucket.artifact_directory, PCLUSTER_S3_ARTIFACTS_DICT.get("config_name")
                     ),
                     "cluster_config_version": self.config.config_version,
-                    "instance_types_data_s3_key": f"{self.bucket.artifact_directory}/configs/instance-types-data.json",
+                    "change_set_s3_key": f"{self.bucket.artifact_directory}/configs/"
+                    f"{PCLUSTER_S3_ARTIFACTS_DICT.get('change_set_name')}",
+                    "instance_types_data_s3_key": f"{self.bucket.artifact_directory}/configs/"
+                    f"{PCLUSTER_S3_ARTIFACTS_DICT.get('instance_types_data_name')}",
                     "custom_node_package": self.config.custom_node_package or "",
                     "custom_awsbatchcli_package": self.config.custom_aws_batch_cli_package or "",
                     "head_node_imds_secured": str(self.config.head_node.imds.secured).lower(),
+                    "compute_node_bootstrap_timeout": get_attr(
+                        self.config, "dev_settings.timeouts.compute_node_bootstrap_timeout", NODE_BOOTSTRAP_TIMEOUT
+                    ),
                     **(
                         get_slurm_specific_dna_json_for_head_node(self.config, self.scheduler_resources)
                         if self._condition_is_slurm()
@@ -1007,6 +1001,7 @@ class ClusterCdkStack(Stack):
                                 "triggers=post.update\n"
                                 "path=Resources.HeadNodeLaunchTemplate.Metadata.AWS::CloudFormation::Init\n"
                                 "action=PATH=/usr/local/bin:/bin:/usr/bin:/opt/aws/bin; "
+                                ". /etc/profile.d/pcluster.sh; "
                                 "cfn-init -v --stack ${StackName} "
                                 "--resource HeadNodeLaunchTemplate --configsets update --region ${Region}\n"
                                 "runas=root\n"
@@ -1083,6 +1078,7 @@ class ClusterCdkStack(Stack):
                 "commands": {
                     "chef": {
                         "command": (
+                            ". /etc/profile.d/pcluster.sh; "
                             "cinc-client --local-mode --config /etc/chef/client.rb --log_level info"
                             " --logfile /var/log/chef-client.log --force-formatter --no-color"
                             " --chef-zero-port 8889 --json-attributes /etc/chef/dna.json"
@@ -1194,7 +1190,7 @@ class ClusterCdkStack(Stack):
 
     def _add_outputs(self):
         # Storage filesystem Ids
-        for storage_type, storage_list in self.shared_storage_mappings.items():
+        for storage_type, storage_list in self.shared_storage_infos.items():
             CfnOutput(
                 self,
                 "{0}Ids".format(storage_type.name),
@@ -1236,8 +1232,8 @@ class ComputeFleetConstruct(Construct):
         cleanup_lambda: awslambda.CfnFunction,
         cleanup_lambda_role: iam.CfnRole,
         compute_security_group: ec2.CfnSecurityGroup,
-        shared_storage_mappings: Dict,
-        shared_storage_options: Dict,
+        shared_storage_infos: Dict,
+        shared_storage_mount_dirs: Dict,
         shared_storage_attributes: Dict,
         compute_node_instance_profiles: Dict[str, str],
         cluster_hosted_zone,
@@ -1249,9 +1245,9 @@ class ComputeFleetConstruct(Construct):
         self._cleanup_lambda_role = cleanup_lambda_role
         self._compute_security_group = compute_security_group
         self._config = cluster_config
-        self._shared_storage_mappings = shared_storage_mappings
+        self._shared_storage_infos = shared_storage_infos
+        self._shared_storage_mount_dirs = shared_storage_mount_dirs
         self._shared_storage_attributes = shared_storage_attributes
-        self._shared_storage_options = shared_storage_options
         self._log_group = log_group
         self._cluster_hosted_zone = cluster_hosted_zone
         self._dynamodb_table = dynamodb_table
@@ -1329,10 +1325,12 @@ class ComputeFleetConstruct(Construct):
             queue_lt_security_groups = get_queue_security_groups_full(self._compute_security_group, queue)
 
             queue_placement_group = None
-            if queue.networking.placement_group and queue.networking.placement_group.enabled:
-                if queue.networking.placement_group.id:
+            if queue.networking.placement_group:
+                if queue.networking.placement_group.id and (
+                    queue.networking.placement_group.enabled or queue.networking.placement_group.is_implied("enabled")
+                ):  # Do not use `PlacementGroup/Id` when `PlacementGroup/Enabled` is explicitly set to `false`
                     queue_placement_group = queue.networking.placement_group.id
-                else:
+                elif queue.networking.placement_group.enabled:
                     queue_placement_group = managed_placement_groups[queue.name].ref
 
             queue_pre_install_action, queue_post_install_action = (None, None)
@@ -1427,8 +1425,11 @@ class ComputeFleetConstruct(Construct):
                         {
                             **{
                                 "EnableEfa": "efa" if compute_resource.efa and compute_resource.efa.enabled else "NONE",
-                                "RAIDOptions": get_shared_storage_options_by_type(
-                                    self._shared_storage_options, SharedStorageType.RAID
+                                "RAIDSharedDir": to_comma_separated_string(
+                                    self._shared_storage_mount_dirs[SharedStorageType.RAID]
+                                ),
+                                "RAIDType": to_comma_separated_string(
+                                    self._shared_storage_attributes[SharedStorageType.RAID]["Type"]
                                 ),
                                 "DisableHyperThreadingManually": "true"
                                 if compute_resource.disable_simultaneous_multithreading_manually
@@ -1446,30 +1447,37 @@ class ComputeFleetConstruct(Construct):
                                 "PostInstallArgs": join_shell_args(queue_post_install_action.args)
                                 if queue_post_install_action and queue_post_install_action.args
                                 else "NONE",
-                                "EFSId": get_shared_storage_ids_by_type(
-                                    self._shared_storage_mappings, SharedStorageType.EFS
+                                "EFSIds": get_shared_storage_ids_by_type(
+                                    self._shared_storage_infos, SharedStorageType.EFS
                                 ),
-                                "EFSOptions": get_shared_storage_options_by_type(
-                                    self._shared_storage_options, SharedStorageType.EFS
+                                "EFSSharedDirs": to_comma_separated_string(
+                                    self._shared_storage_mount_dirs[SharedStorageType.EFS]
                                 ),
-                                "FSXId": get_shared_storage_ids_by_type(
-                                    self._shared_storage_mappings, SharedStorageType.FSX
+                                "FSXIds": get_shared_storage_ids_by_type(
+                                    self._shared_storage_infos, SharedStorageType.FSX
                                 ),
-                                "FSXMountName": self._shared_storage_attributes[SharedStorageType.FSX].get(
-                                    "MountName", ""
+                                "FSXMountNames": to_comma_separated_string(
+                                    self._shared_storage_attributes[SharedStorageType.FSX]["MountNames"]
                                 ),
-                                "FSXDNSName": self._shared_storage_attributes[SharedStorageType.FSX].get("DNSName", ""),
-                                "FSXOptions": get_shared_storage_options_by_type(
-                                    self._shared_storage_options, SharedStorageType.FSX
+                                "FSXDNSNames": to_comma_separated_string(
+                                    self._shared_storage_attributes[SharedStorageType.FSX]["DNSNames"]
+                                ),
+                                "FSXVolumeJunctionPaths": to_comma_separated_string(
+                                    self._shared_storage_attributes[SharedStorageType.FSX]["VolumeJunctionPaths"]
+                                ),
+                                "FSXFileSystemTypes": to_comma_separated_string(
+                                    self._shared_storage_attributes[SharedStorageType.FSX]["FileSystemTypes"]
+                                ),
+                                "FSXSharedDirs": to_comma_separated_string(
+                                    self._shared_storage_mount_dirs[SharedStorageType.FSX]
                                 ),
                                 "Scheduler": self._config.scheduling.scheduler,
                                 "EphemeralDir": queue.compute_settings.local_storage.ephemeral_volume.mount_dir
-                                if queue.compute_settings
-                                and queue.compute_settings.local_storage
+                                if isinstance(queue, (SlurmQueue, SchedulerPluginQueue))
                                 and queue.compute_settings.local_storage.ephemeral_volume
-                                else "/scratch",
-                                "EbsSharedDirs": get_shared_storage_options_by_type(
-                                    self._shared_storage_options, SharedStorageType.EBS
+                                else DEFAULT_EPHEMERAL_DIR,
+                                "EbsSharedDirs": to_comma_separated_string(
+                                    self._shared_storage_mount_dirs[SharedStorageType.EBS]
                                 ),
                                 "ClusterDNSDomain": str(self._cluster_hosted_zone.name)
                                 if self._cluster_hosted_zone
@@ -1478,7 +1486,7 @@ class ComputeFleetConstruct(Construct):
                                 if self._cluster_hosted_zone
                                 else "",
                                 "OSUser": OS_MAPPING[self._config.image.os]["user"],
-                                "DynamoDBTable": self._dynamodb_table.ref if self._dynamodb_table else "NONE",
+                                "SlurmDynamoDBTable": self._dynamodb_table.ref if self._dynamodb_table else "NONE",
                                 "LogGroupName": self._log_group.log_group_name
                                 if self._config.monitoring.logs.cloud_watch.enabled
                                 else "NONE",
@@ -1497,6 +1505,13 @@ class ComputeFleetConstruct(Construct):
                                 ).lower(),
                                 "HeadNodePrivateIp": self._head_eni.attr_primary_private_ip_address,
                                 "DirectoryServiceEnabled": str(self._config.directory_service is not None).lower(),
+                                "Timeout": str(
+                                    get_attr(
+                                        self._config,
+                                        "dev_settings.timeouts.compute_node_bootstrap_timeout",
+                                        NODE_BOOTSTRAP_TIMEOUT,
+                                    )
+                                ),
                             },
                             **get_common_user_data_env(queue, self._config),
                         },
@@ -1507,7 +1522,7 @@ class ComputeFleetConstruct(Construct):
                     ec2.CfnLaunchTemplate.TagSpecificationProperty(
                         resource_type="instance",
                         tags=get_default_instance_tags(
-                            self.stack_name, self._config, compute_resource, "Compute", self._shared_storage_mappings
+                            self.stack_name, self._config, compute_resource, "Compute", self._shared_storage_infos
                         )
                         + [CfnTag(key=PCLUSTER_QUEUE_NAME_TAG, value=queue.name)]
                         + [CfnTag(key=PCLUSTER_COMPUTE_RESOURCE_NAME_TAG, value=compute_resource.name)]
