@@ -8,13 +8,13 @@
 # or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 # OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
 # limitations under the License.
-from collections import namedtuple
+from collections import defaultdict, namedtuple
 
 from aws_cdk import aws_cloudwatch as cloudwatch
 from aws_cdk import aws_ec2 as ec2
 from aws_cdk.core import Construct, Stack
 
-from pcluster.config.cluster_config import BaseClusterConfig, SharedStorageType
+from pcluster.config.cluster_config import BaseClusterConfig, SharedFsxLustre, SharedStorageType
 
 MAX_WIDTH = 24
 
@@ -49,7 +49,7 @@ class CWDashboardConstruct(Construct):
         id: str,
         cluster_config: BaseClusterConfig,
         head_node_instance: ec2.CfnInstance,
-        shared_storage_mappings: dict,
+        shared_storage_infos: dict,
         cw_log_group_name: str,
     ):
         super().__init__(scope, id)
@@ -57,7 +57,7 @@ class CWDashboardConstruct(Construct):
         self.stack_name = stack_name
         self.config = cluster_config
         self.head_node_instance = head_node_instance
-        self.shared_storage_mappings = shared_storage_mappings
+        self.shared_storage_infos = shared_storage_infos
         self.cw_log_group_name = cw_log_group_name
 
         self.dashboard_name = self.stack_name + "-" + self._stack_region
@@ -121,15 +121,15 @@ class CWDashboardConstruct(Construct):
             (SharedStorageType.EBS, "## EBS Metrics"),
             (SharedStorageType.RAID, "## RAID Metrics"),
         ]:
-            if len(self.shared_storage_mappings[storage_type]) > 0:
+            if len(self.shared_storage_infos[storage_type]) > 0:
                 self._add_volume_metrics_graphs(title, storage_type, ebs_metrics, conditional_ebs_metrics)
 
         # Add EFS metrics graphs
-        if len(self.shared_storage_mappings[SharedStorageType.EFS]) > 0:
+        if len(self.shared_storage_infos[SharedStorageType.EFS]) > 0:
             self._add_efs_metrics_graphs()
 
         # Add FSx metrics graphs
-        if len(self.shared_storage_mappings[SharedStorageType.FSX]) > 0:
+        if len(self.shared_storage_infos[SharedStorageType.FSX]) > 0:
             self._add_fsx_metrics_graphs()
 
         # Head Node logs, if CW Logs are enabled
@@ -159,6 +159,10 @@ class CWDashboardConstruct(Construct):
                 for cond_dict in log_params.conditions:
                     if cond_dict.param in cond_dict.allowed_values:
                         self.empty_section = False
+                        return
+            else:
+                self.empty_section = False
+                return
 
     def _add_text_widget(self, markdown):
         """Add the textwidget to the cloudwatch dashboard and update coordinates."""
@@ -231,6 +235,57 @@ class CWDashboardConstruct(Construct):
             widgets_list.append(graph_widget)
         return widgets_list
 
+    def _add_fsx_widgets(self, storages_list):
+        common_metrics = [
+            new_pcluster_metric(title="Data Read/Write Ops", metrics=["DataReadOperations", "DataWriteOperations"]),
+            new_pcluster_metric(title="Data Read/Write Bytes", metrics=["DataReadBytes", "DataWriteBytes"]),
+        ]
+        free_data_storage_capacity_graph_title = "Free Data Storage Capacity"
+        lustre_metrics = common_metrics + [
+            new_pcluster_metric(title=free_data_storage_capacity_graph_title, metrics=["FreeDataStorageCapacity"])
+        ]
+        namespace = "AWS/FSx"
+        metric_graphs = defaultdict(list)
+        for index, storage in enumerate(storages_list):
+            if isinstance(storage.config, SharedFsxLustre):
+                metrics = lustre_metrics
+                dimensions_map = {"FileSystemId": storage.id}
+            else:
+                metrics = common_metrics
+                dimensions_map = {"FileSystemId": storage.config.file_system_id, "VolumeId": storage.id}
+
+                # FSx Ontap/OpenZFS do not provide free capacity metric. The code below generates the metric using math.
+                free_capacity_metric = cloudwatch.MathExpression(
+                    label=f"{storage.id} FreeDataStorageCapacity",
+                    expression=f"capacity{index} - used_capacity{index}",
+                    using_metrics={
+                        f"capacity{index}": cloudwatch.Metric(
+                            namespace=namespace,
+                            metric_name="StorageCapacity",
+                            dimensions_map=dimensions_map,
+                        ),
+                        f"used_capacity{index}": cloudwatch.Metric(
+                            namespace=namespace,
+                            metric_name="UsedStorageCapacity",
+                            dimensions_map=dimensions_map,
+                        ),
+                    },
+                )
+                metric_graphs[free_data_storage_capacity_graph_title].append(free_capacity_metric)
+            for metrics_param in metrics:
+                metric_list = metric_graphs[metrics_param.title]
+                for metric in metrics_param.metrics:
+                    cloudwatch_metric = cloudwatch.Metric(
+                        namespace=namespace,
+                        metric_name=metric,
+                        dimensions_map=dimensions_map,
+                    )
+                    metric_list.append(cloudwatch_metric)
+        widgets_list = []
+        for title, metric_list in metric_graphs.items():
+            widgets_list.append(self._generate_graph_widget(title, metric_list))
+        return widgets_list
+
     def _add_head_node_instance_metrics_graphs(self):
         # Create a text widget for subtitle "Head Node Instance Metrics"
         self._add_text_widget("## Head Node Instance Metrics")
@@ -257,7 +312,7 @@ class CWDashboardConstruct(Construct):
         self._add_text_widget(title)
 
         # Get a list of volumes
-        volumes_list = self.shared_storage_mappings[storage_type]
+        volumes_list = self.shared_storage_infos[storage_type]
 
         # Unconditional EBS metrics
         widgets_list = self._add_storage_widgets(
@@ -279,7 +334,7 @@ class CWDashboardConstruct(Construct):
 
     def _add_efs_metrics_graphs(self):
         self._add_text_widget("## EFS Metrics")
-        efs_volumes_list = self.shared_storage_mappings[SharedStorageType.EFS]
+        efs_volumes_list = self.shared_storage_infos[SharedStorageType.EFS]
 
         # Unconditional EFS metrics
         efs_metrics = [
@@ -313,21 +368,11 @@ class CWDashboardConstruct(Construct):
 
     def _add_fsx_metrics_graphs(self):
         self._add_text_widget("## FSx Metrics")
-        fsx_volumes_list = self.shared_storage_mappings[SharedStorageType.FSX]
-
-        # Unconditional FSX metrics
-        fsx_metrics = [
-            new_pcluster_metric(title="Data Read/Write Ops", metrics=["DataReadOperations", "DataWriteOperations"]),
-            new_pcluster_metric(title="Data Read/Write Bytes", metrics=["DataReadBytes", "DataWriteBytes"]),
-            new_pcluster_metric(title="Free Data Storage Capacity", metrics=["FreeDataStorageCapacity"]),
-        ]
+        fsx_volumes_list = self.shared_storage_infos[SharedStorageType.FSX]
 
         # Add FSx metrics graphs and update coordinates
-        widgets_list = self._add_storage_widgets(
-            metrics=fsx_metrics,
+        widgets_list = self._add_fsx_widgets(
             storages_list=fsx_volumes_list,
-            namespace="AWS/FSx",
-            dimension_name="FileSystemId",
         )
         self.cloudwatch_dashboard.add_widgets(*widgets_list)
         self._update_coord_after_section(self.graph_height)
