@@ -8,15 +8,19 @@
 # or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 # OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
 # limitations under the License.
+import math
 import re
 from abc import ABC
+from collections import defaultdict
 from enum import Enum
+from itertools import combinations, product
 
 from pcluster.aws.aws_api import AWSApi
 from pcluster.aws.common import AWSClientError
 from pcluster.cli.commands.dcv_util import get_supported_dcv_os
 from pcluster.constants import (
     CIDR_ALL_IPS,
+    FSX_PORTS,
     PCLUSTER_IMAGE_BUILD_STATUS_TAG,
     PCLUSTER_NAME_MAX_LENGTH,
     PCLUSTER_NAME_REGEX,
@@ -25,6 +29,7 @@ from pcluster.constants import (
     SCHEDULERS_SUPPORTING_IMDS_SECURED,
     SUPPORTED_OSES,
     SUPPORTED_REGIONS,
+    SUPPORTED_SCHEDULERS,
 )
 from pcluster.utils import get_installed_version, get_supported_os_for_architecture, get_supported_os_for_scheduler
 from pcluster.validators.common import FailureLevel, Validator
@@ -154,23 +159,6 @@ class ComputeResourceSizeValidator(Validator):
             self._add_failure("Max count must be greater than or equal to min count.", FailureLevel.ERROR)
 
 
-class DisableSimultaneousMultithreadingArchitectureValidator(Validator):
-    """
-    Simultaneous Multithreading architecture validator.
-
-    Validate Simultaneous Multithreading and architecture combination.
-    """
-
-    def _validate(self, disable_simultaneous_multithreading, architecture: str):
-        supported_architectures = ["x86_64"]
-        if disable_simultaneous_multithreading and architecture not in supported_architectures:
-            self._add_failure(
-                "Disabling simultaneous multithreading is only supported on instance types that support "
-                "these architectures: {0}.".format(", ".join(supported_architectures)),
-                FailureLevel.ERROR,
-            )
-
-
 class EfaOsArchitectureValidator(Validator):
     """OS and architecture combination validator if EFA is enabled."""
 
@@ -181,6 +169,34 @@ class EfaOsArchitectureValidator(Validator):
             )
 
 
+class SchedulableMemoryValidator(Validator):
+    """Validate SchedulableMemory parameter passed by user."""
+
+    def _validate(self, schedulable_memory, ec2memory, instance_type):
+        if schedulable_memory is not None:
+            if schedulable_memory < 1:
+                self._add_failure("SchedulableMemory must be at least 1 MiB.", FailureLevel.ERROR)
+            if ec2memory is None:
+                self._add_failure(
+                    f"SchedulableMemory was set but EC2 memory is not available for selected instance type "
+                    f"{instance_type}. Defaulting to 1 MiB.",
+                    FailureLevel.WARNING,
+                )
+            else:
+                if schedulable_memory > ec2memory:
+                    self._add_failure(
+                        f"SchedulableMemory cannot be larger than EC2 Memory for selected instance type "
+                        f"{instance_type} ({ec2memory} MiB).",
+                        FailureLevel.ERROR,
+                    )
+                if schedulable_memory < math.floor(0.95 * ec2memory):
+                    self._add_failure(
+                        f"SchedulableMemory was set lower than 95% of EC2 Memory for selected instance type "
+                        f"{instance_type} ({ec2memory} MiB).",
+                        FailureLevel.INFO,
+                    )
+
+
 class ArchitectureOsValidator(Validator):
     """
     Validate OS and architecture combination.
@@ -276,7 +292,11 @@ class EfaValidator(Validator):
             self._add_failure(f"Instance type '{instance_type}' does not support EFA.", FailureLevel.ERROR)
         if instance_type_supports_efa and not efa_enabled:
             self._add_failure(
-                f"Instance type '{instance_type}' supports EFA, but it is not enabled.", FailureLevel.WARNING
+                f"The EC2 instance selected ({instance_type}) supports enhanced networking capabilities using "
+                "Elastic Fabric Adapter (EFA). EFA enables you to run applications requiring high levels of "
+                "inter-node communications at scale on AWS at no additional charge. You can update the cluster's "
+                "configuration to enable EFA (https://docs.aws.amazon.com/parallelcluster/latest/ug/efa-v3.html)",
+                FailureLevel.WARNING,
             )
         if gdr_support and not efa_enabled:
             self._add_failure("The EFA GDR Support can be used only if EFA is enabled.", FailureLevel.ERROR)
@@ -285,7 +305,13 @@ class EfaValidator(Validator):
 class EfaPlacementGroupValidator(Validator):
     """Validate placement group if EFA is enabled."""
 
-    def _validate(self, efa_enabled, placement_group_enabled, placement_group_config_implicit):
+    def _validate(self, efa_enabled, placement_group):
+        placement_group_enabled = placement_group and (
+            placement_group.enabled or (placement_group.id and placement_group.is_implied("enabled"))
+        )
+        placement_group_config_implicit = placement_group is None or (
+            placement_group.is_implied("enabled") and placement_group.id is None
+        )
         if efa_enabled and placement_group_config_implicit:
             self._add_failure(
                 "The placement group for EFA-enabled compute resources must be explicit. "
@@ -345,46 +371,49 @@ class EfaSecurityGroupValidator(Validator):
 # --------------- Storage validators --------------- #
 
 
-def _check_in_out_access(security_groups_ids, port):
+def _check_in_out_access(security_groups_ids, port, is_cidr_optional, protocol="tcp"):
     """
     Verify given list of security groups to check if they allow in and out access on the given port.
 
     :param security_groups_ids: list of security groups to verify
     :param port: port to verify
+    :param is_cidr_optional: if it is True, don't enforce check on CIDR.
+    :param protocol: the IP protocol to be checked.
     :return: True if both in and out access are allowed
     :raise: ClientError if a given security group doesn't exist
     """
-    in_out_access = False
     in_access = False
     out_access = False
 
-    for sec_group in AWSApi.instance().ec2.describe_security_groups(security_groups_ids).get("SecurityGroups"):
+    for sec_group in AWSApi.instance().ec2.describe_security_groups(security_groups_ids):
 
         # Check all inbound rules
         for rule in sec_group.get("IpPermissions"):
-            if _check_sg_rules_for_port(rule, port):
-                in_access = True
-                break
+            if _check_sg_rules_for_port(rule, port, protocol):
+                if is_cidr_optional or rule.get("IpRanges") or rule.get("PrefixListIds"):
+                    in_access = True
+                    break
 
         # Check all outbound rules
         for rule in sec_group.get("IpPermissionsEgress"):
-            if _check_sg_rules_for_port(rule, port):
-                out_access = True
-                break
+            if _check_sg_rules_for_port(rule, port, protocol):
+                if is_cidr_optional or rule.get("IpRanges") or rule.get("PrefixListIds"):
+                    out_access = True
+                    break
 
         if in_access and out_access:
-            in_out_access = True
-            break
+            return True
 
-    return in_out_access
+    return False
 
 
-def _check_sg_rules_for_port(rule, port_to_check):
+def _check_sg_rules_for_port(rule, port_to_check, protocol):
     """
     Verify if the security group rule accepts connections on the given port.
 
     :param rule: The rule to check
     :param port_to_check: The port to check
+    :param protocol: the IP protocol to be checked.
     :return: True if the rule accepts connection, False otherwise
     """
     from_port = rule.get("FromPort")
@@ -394,65 +423,113 @@ def _check_sg_rules_for_port(rule, port_to_check):
     # if ip_protocol is -1, all ports are allowed
     if ip_protocol == "-1":
         return True
-    # tcp == protocol 6,
-    # if the ip_protocol is tcp, from_port and to_port must >= 0 and <= 65535
-    if (ip_protocol in ["tcp", "6"]) and (from_port <= port_to_check <= to_port):
+    # Add protocol number in addition to the protocol name
+    if protocol == "tcp":
+        expected_protocol = [protocol, "6"]
+    elif protocol == "udp":
+        expected_protocol = [protocol, "17"]
+    else:
+        # ToDo: When adding new checks for other protocols, change the code to include the protocol number too.
+        expected_protocol = [protocol]
+
+    if (ip_protocol in expected_protocol) and (from_port <= port_to_check <= to_port):
         return True
 
     return False
 
 
-class FsxNetworkingValidator(Validator):
+class ExistingFsxNetworkingValidator(Validator):
     """
     FSx networking validator.
 
     Validate file system mount point according to the head node subnet.
+    The reason to have this structure is to make boto3 calls as few as possible.
     """
 
-    def _validate(self, file_system_id, head_node_subnet_id):
-        try:
+    def _describe_network_interfaces(self, file_systems):
+        all_network_interfaces = []
+        for file_system in file_systems:
+            all_network_interfaces.extend(file_system.network_interface_ids)
+        if all_network_interfaces:
+            response = AWSApi.instance().ec2.describe_network_interfaces(all_network_interfaces)
+            network_interfaces_data = {}
+            for network_interface in response:
+                network_interfaces_data[network_interface["NetworkInterfaceId"]] = network_interface
+            return network_interfaces_data
+        else:
+            return {}
 
+    def _validate(self, file_system_ids, head_node_subnet_id, are_all_security_groups_customized):
+        try:
             # Check to see if there is any existing mt on the fs
-            file_system = AWSApi.instance().fsx.get_filesystem_info(file_system_id).file_system_data
+            file_systems = AWSApi.instance().fsx.get_file_systems_info(file_system_ids)
 
             vpc_id = AWSApi.instance().ec2.get_subnet_vpc(head_node_subnet_id)
 
+            network_interfaces_data = self._describe_network_interfaces(file_systems)
+
+            self._check_file_systems(are_all_security_groups_customized, file_systems, network_interfaces_data, vpc_id)
+        except AWSClientError as e:
+            self._add_failure(str(e), FailureLevel.ERROR)
+
+    def _check_file_systems(self, are_all_security_groups_customized, file_systems, network_interfaces_data, vpc_id):
+        for file_system in file_systems:
             # Check to see if fs is in the same VPC as the stack
-            if file_system.get("VpcId") != vpc_id:
+            file_system_id = file_system.file_system_id
+            if file_system.vpc_id != vpc_id:
                 self._add_failure(
                     "Currently only support using FSx file system that is in the same VPC as the cluster. "
-                    "The file system provided is in {0}.".format(file_system.get("VpcId")),
+                    "The file system provided is in {0}.".format(file_system.vpc_id),
                     FailureLevel.ERROR,
                 )
 
-            # If there is an existing mt in the az, need to check the inbound and outbound rules of the security groups
-            network_interface_ids = file_system.get("NetworkInterfaceIds")
+            # If there is an existing mt in the az, check the inbound and outbound rules of the security groups
+            network_interface_ids = file_system.network_interface_ids
             if not network_interface_ids:
                 self._add_failure(
-                    "Unable to validate FSx security groups. The given FSx file system '{0}' doesn't have "
-                    "Elastic Network Interfaces attached to it.".format(file_system_id),
+                    f"Unable to validate FSx security groups. The given FSx file system '{file_system_id}'"
+                    " doesn't have Elastic Network Interfaces attached to it.",
                     FailureLevel.ERROR,
                 )
             else:
-                network_interface_responses = AWSApi.instance().ec2.describe_network_interfaces(network_interface_ids)
+                network_interface_responses = []
+                for network_interface_id in network_interface_ids:
+                    network_interface_responses.append(network_interfaces_data[network_interface_id])
 
-                fs_access = False
                 network_interfaces = [ni for ni in network_interface_responses if ni.get("VpcId") == vpc_id]
-                for network_interface in network_interfaces:
-                    # Get list of security group IDs
-                    sg_ids = [sg.get("GroupId") for sg in network_interface.get("Groups")]
-                    if _check_in_out_access(sg_ids, port=988):
-                        fs_access = True
-                        break
-                if not fs_access:
-                    self._add_failure(
-                        "The current security group settings on file system '{0}' does not satisfy mounting requirement"
-                        ". The file system must be associated to a security group that allows inbound and outbound "
-                        "TCP traffic through port 988.".format(file_system_id),
-                        FailureLevel.ERROR,
+
+                for protocol, ports in FSX_PORTS[file_system.file_system_type].items():
+                    missing_ports = self._get_missing_ports(
+                        are_all_security_groups_customized, network_interfaces, ports, protocol
                     )
-        except AWSClientError as e:
-            self._add_failure(str(e), FailureLevel.ERROR)
+
+                    if missing_ports:
+                        self._add_failure(
+                            f"The current security group settings on file system '{file_system_id}' does not"
+                            " satisfy mounting requirement. The file system must be associated to a security group"
+                            f" that allows inbound and outbound {protocol.upper()} traffic through ports {ports}. "
+                            f"Missing ports: {missing_ports}",
+                            FailureLevel.ERROR,
+                        )
+
+    def _get_missing_ports(self, are_all_security_groups_customized, network_interfaces, ports, protocol):
+        missing_ports = []
+        for port in ports:
+            fs_access = False
+            for network_interface in network_interfaces:
+                # Get list of security group IDs
+                sg_ids = [sg.get("GroupId") for sg in network_interface.get("Groups")]
+                if _check_in_out_access(
+                    sg_ids,
+                    port=port,
+                    is_cidr_optional=are_all_security_groups_customized,
+                    protocol=protocol,
+                ):
+                    fs_access = True
+                    break
+            if not fs_access:
+                missing_ports.append(port)
+        return missing_ports
 
 
 class FsxArchitectureOsValidator(Validator):
@@ -493,13 +570,15 @@ def _find_duplicate_params(param_list):
     return duplicated_params
 
 
-def _find_overlapping_paths(paths_list):
+def _find_overlapping_paths(shared_paths_list, local_paths_list):
     overlapping_paths = []
-    if paths_list:
-        for path in paths_list:
-            is_overlapping = any(x for x in paths_list if x != path and x.startswith(path + "/"))
+    if shared_paths_list:
+        for path1, path2 in list(combinations(shared_paths_list, 2)) + list(
+            product(shared_paths_list, local_paths_list)
+        ):  # Check all pairs in shared paths list and all pairs between shared paths list and local paths list
+            is_overlapping = path1.startswith(path2 + "/") or path2.startswith(path1 + "/")
             if is_overlapping:
-                overlapping_paths.append(path)
+                overlapping_paths.extend([path1, path2])
 
     return overlapping_paths
 
@@ -511,32 +590,46 @@ class DuplicateMountDirValidator(Validator):
     Verify if there are duplicated mount dirs between shared storage and ephemeral volumes.
     """
 
-    def _validate(self, mount_dir_list):
-        duplicated_mount_dirs = _find_duplicate_params(mount_dir_list)
-        if duplicated_mount_dirs:
-            self._add_failure(
-                "Mount {0} {1} cannot be specified for multiple file systems".format(
-                    "directories" if len(duplicated_mount_dirs) > 1 else "directory",
-                    ", ".join(mount_dir for mount_dir in duplicated_mount_dirs),
-                ),
-                FailureLevel.ERROR,
-            )
+    def _validate(self, shared_storage_name_mount_dir_tuple_list, local_mount_dir_instance_types_dict):
+        mount_dir_to_names = defaultdict(list)
+        for shared_storage_name, shared_mount_dir in shared_storage_name_mount_dir_tuple_list:
+            mount_dir_to_names[shared_mount_dir].append(shared_storage_name)
+        for mount_dir, names in mount_dir_to_names.items():
+            if len(names) > 1:
+                self._add_failure(
+                    f"The mount directory `{mount_dir}` is used for multiple shared storage: {names}. "
+                    "Shared storage mount directories should be unique. "
+                    "Please change the mount directory configuration of the shared storage.",
+                    FailureLevel.ERROR,
+                )
+        for local_mount_dir, instance_types in local_mount_dir_instance_types_dict.items():
+            shared_storage_names = mount_dir_to_names.get(local_mount_dir)
+            if shared_storage_names:
+                self._add_failure(
+                    f"The mount directory `{local_mount_dir}` used for shared storage {shared_storage_names} "
+                    f"clashes with the one used for ephemeral volumes of the instances {list(instance_types)}. "
+                    f"Please change the mount directory configuration of either the shared storage or the ephemeral "
+                    f"volume of the impacted nodes.",
+                    FailureLevel.WARNING,
+                )
 
 
 class OverlappingMountDirValidator(Validator):
     """
     Mount dir validator.
 
-    Verify if there are overlapping mount dirs between shared storage and ephemeral volumes.
+    Verify if there are overlap mount dirs.
+    1. Shared storage directories can not overlap with each other.
+    2. Shared storage directories can not overlap with ephemeral storage directories.
+    3. Ephemeral storage directories can overlap with each other, because they are local to compute nodes.
     Two mount dirs are overlapped if one is contained into the other.
     """
 
-    def _validate(self, mount_dir_list):
-        overlapping_mount_dirs = _find_overlapping_paths(mount_dir_list)
+    def _validate(self, shared_mount_dir_list, local_mount_dir_list):
+        overlapping_mount_dirs = _find_overlapping_paths(shared_mount_dir_list, local_mount_dir_list)
         if overlapping_mount_dirs:
             self._add_failure(
-                "Mount {0} {1} cannot contain other mount directories".format(
-                    "directories" if len(overlapping_mount_dirs) > 1 else "directory",
+                "Mount directories {0} cannot overlap".format(
                     ", ".join(mount_dir for mount_dir in overlapping_mount_dirs),
                 ),
                 FailureLevel.ERROR,
@@ -553,8 +646,8 @@ class NumberOfStorageValidator(Validator):
     def _validate(self, storage_type: str, max_number: int, storage_count: int):
         if storage_count > max_number:
             self._add_failure(
-                f"Invalid number of shared storage of {storage_type} type specified. "
-                f"Currently only supports upto {max_number}.",
+                f"Too many {storage_type} shared storage specified in the configuration. "
+                f"ParallelCluster supports {max_number} {storage_type}.",
                 FailureLevel.ERROR,
             )
 
@@ -566,22 +659,22 @@ class EfsIdValidator(Validator):  # TODO add tests
     Validate if there are existing mount target in the head node availability zone
     """
 
-    def _validate(self, efs_id, head_node_avail_zone: str):
-        # Get head node availability zone
-        head_node_target_id = AWSApi.instance().efs.get_efs_mount_target_id(efs_id, head_node_avail_zone)
-        # If there is an existing mt in the az, need to check the inbound and outbound rules of the security groups
-        if head_node_target_id:
-            # Get list of security group IDs of the mount target
-            sg_ids = AWSApi.instance().efs.get_efs_mount_target_security_groups(head_node_target_id)
-            if not _check_in_out_access(sg_ids, port=2049):
-                self._add_failure(
-                    "There is an existing Mount Target {0} in the Availability Zone {1} for EFS {2}, "
-                    "but it does not have a security group that allows inbound and outbound rules to support NFS. "
-                    "Please modify the Mount Target's security group, to allow traffic on port 2049.".format(
-                        head_node_target_id, head_node_avail_zone, efs_id
-                    ),
-                    FailureLevel.WARNING,
-                )
+    def _validate(self, efs_id, avail_zones: set, are_all_security_groups_customized):
+        for avail_zone in avail_zones:
+            head_node_target_id = AWSApi.instance().efs.get_efs_mount_target_id(efs_id, avail_zone)
+            # If there is an existing mt in the az, need to check the inbound and outbound rules of the security groups
+            if head_node_target_id:
+                # Get list of security group IDs of the mount target
+                sg_ids = AWSApi.instance().efs.get_efs_mount_target_security_groups(head_node_target_id)
+                if not _check_in_out_access(sg_ids, port=2049, is_cidr_optional=are_all_security_groups_customized):
+                    self._add_failure(
+                        "There is an existing Mount Target {0} in the Availability Zone {1} for EFS {2}, "
+                        "but it does not have a security group that allows inbound and outbound rules to support NFS. "
+                        "Please modify the Mount Target's security group, to allow traffic on port 2049.".format(
+                            head_node_target_id, avail_zone, efs_id
+                        ),
+                        FailureLevel.ERROR,
+                    )
 
 
 class SharedStorageNameValidator(Validator):
@@ -613,6 +706,44 @@ class SharedStorageNameValidator(Validator):
             self._add_failure(f"It is forbidden to use '{name}' as a name.", FailureLevel.ERROR)
 
 
+class SharedStorageMountDirValidator(Validator):
+    """
+    Shared storage mount directory validator.
+
+    Make sure the mount directory is not the same as any reserved directory.
+    """
+
+    def _validate(self, mount_dir: str):
+        reserved_directories = [
+            "/bin",
+            "/boot",
+            "/dev",
+            "/etc",
+            "/home",
+            "/lib",
+            "/lib64",
+            "/media",
+            "/mnt",
+            "/opt",
+            "/proc",
+            "/root",
+            "/run",
+            "/sbin",
+            "/srv",
+            "/sys",
+            "/tmp",  # nosec nosemgrep
+            "/usr",
+            "/var",
+        ]
+        if not mount_dir.startswith("/"):
+            mount_dir = "/" + mount_dir
+        if mount_dir in reserved_directories:
+            self._add_failure(
+                f"Error: The shared storage mount directory {mount_dir} is reserved. Please use another directory",
+                FailureLevel.ERROR,
+            )
+
+
 # --------------- Third party software validators --------------- #
 
 
@@ -970,6 +1101,19 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
         )
 
 
+class RootVolumeSizeValidator(Validator):
+    """Verify the root volume size is equal or greater to the size of the snapshot of the AMI."""
+
+    def _validate(self, root_volume_size, ami_volume_size):
+        if root_volume_size:
+            if root_volume_size < ami_volume_size:
+                self._add_failure(
+                    f"Root volume size {root_volume_size} GiB must be equal or greater than the volume size of "
+                    f"the AMI: {ami_volume_size} GiB.",
+                    FailureLevel.ERROR,
+                )
+
+
 class HostedZoneValidator(Validator):
     """Validate custom private domain in the same VPC as head node."""
 
@@ -1002,3 +1146,14 @@ class HostedZoneValidator(Validator):
                 ),
                 FailureLevel.ERROR,
             )
+
+
+class SchedulerValidator(Validator):
+    """Validate that only supported schedulers are specified."""
+
+    def _validate(self, scheduler):
+        if scheduler not in SUPPORTED_SCHEDULERS:
+            self._add_failure(
+                f"{scheduler} scheduler is not supported. Supported schedulers are: {', '.join(SUPPORTED_SCHEDULERS)}.",
+                FailureLevel.ERROR,
+            )
